// GMatrixEvaluation.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"

#include <windows.h>

#include "..\..\Gmatrix.h"

GMATRIX_DECLARE(GlobalMatrix, 5, 3);

#define Pause() {printf("\nPause. Press any key to continue.");getchar();}

void Eval_Rank(void)
{
  GMATRIX_DECLARE(A, 6, 6);
  GMATRIX_DECLARE(U, 6, 6);
  GMATRIX_DECLARE(S, 6, 6);
  GMATRIX_DECLARE(V, 6, 6);

  GMATRIX_SETSIZE(A,5,5);
  GMATRIX_ORTHOGONAL(A,1);
  printf("\n**** GMATRIX_RANK of an orthogonal matrix");
  GMATRIX_PRINT(A);
  printf("\n rank = %i", GMATRIX_RANK(A));
  Pause();

  GMATRIX_SETSIZE(A,3,6);
  GMATRIX_RANDN(A);
  printf("\n**** GMATRIX_RANK of a random matrix");
  GMATRIX_PRINT(A);
  printf("\n rank = %i", GMATRIX_RANK(A));
  Pause();

  GMATRIX_SETSIZE(A,3,3);
  GMATRIX_WILKINSON(A);
  printf("\n**** GMATRIX_RANK of a Wilkinson matrix");
  GMATRIX_PRINT(A);
  printf("\n rank = %i", GMATRIX_RANK(A));

  GMATRIX_SETSIZE(A,3,3);
  GMATRIX_IDENTITY(A);
  GMATRIX_DATA(A,1,1) = 1e-20;
  printf("\n**** GMATRIX_RANK of a ill-conditioned matrix");
  GMATRIX_PRINT(A);
  printf("\n rank = %i", GMATRIX_RANK(A));

//  GMATRIX_SVD(U,S,V,A,TRUE);
//  GMATRIX_PRINT(S);
  Pause();

  return;
}

void Eval_Swap(void)
{
  GMATRIX_DECLARE(A, 6, 6);

  GMATRIX_SETSIZE(A,3,4);
  GMATRIX_RANDN(A);
  GMATRIX_PRINT(A);
  GMATRIX_SWAP_ROW(A,3,2);
  GMATRIX_PRINT(A);
  GMATRIX_SWAP_COLUMN(A,3,2);
  GMATRIX_PRINT(A);
  Pause();
  return;
}

void Eval_Pseudoinverse(void)
{
  GMATRIX_DECLARE(A, 6, 6);
  GMATRIX_DECLARE(Apinv, 6, 6);
  GMATRIX_DECLARE(MatAux1, 6, 6);
  GMATRIX_DECLARE(MatAux2, 6, 6);
  
  GMATRIX_SETSIZE(A,5,5);
  GMATRIX_WILKINSON(A);
  GMATRIX_PSEUDOINVERSE(Apinv,A,MatAux1);
  printf("\n**** GMATRIX_PSEUDOINVERSE of a Wilkinson matrix");
  GMATRIX_PRINT(Apinv);

  Pause();
 
  GMATRIX_SETSIZE(A,4,4);
  GMATRIX_ORTHOGONAL(A,1);
  GMATRIX_PSEUDOINVERSE(Apinv,A,MatAux1);
  printf("\n**** GMATRIX_PSEUDOINVERSE of an orthogonal matrix");
  GMATRIX_PRINT(Apinv);

  Pause();
 
  GMATRIX_SETSIZE(A,4,2);
  GMATRIX_RANDN(A);
  GMATRIX_PSEUDOINVERSE(Apinv,A,MatAux1);
  printf("\n**** GMATRIX_PSEUDOINVERSE of a random matrix 4x2");
  GMATRIX_PRINT(Apinv);
  

  Pause();

  GMATRIX_SETSIZE(A,2,4);
  GMATRIX_RANDN(A);
  GMATRIX_PSEUDOINVERSE(Apinv,A,MatAux1);
  printf("\n**** GMATRIX_PSEUDOINVERSE of a random matrix 2x4");
  GMATRIX_PRINT(Apinv);

}

void Eval_SVD(void)
{
  GMATRIX_DECLARE(A, 6, 6);
  GMATRIX_DECLARE(U, 6, 6);
  GMATRIX_DECLARE(S, 6, 6);
  GMATRIX_DECLARE(V, 6, 6);
  GMATRIX_DECLARE(MatAux1, 6, 6);
  GMATRIX_DECLARE(MatAux2, 6, 6);
  
  GMATRIX_SETSIZE(A,3,3);
  GMATRIX_WILKINSON(A);
  GMATRIX_SVD(U,S,V,A,TRUE);
  printf("\n**** GMATRIX_SVD of a Wilkinson matrix");
  GMATRIX_PRINT(A);
  GMATRIX_PRINT(U);
  GMATRIX_PRINT(V);
  GMATRIX_PRINT(S);
  GMATRIX_MULTIPLY_COPY_EXTENDED(MatAux1,S,FALSE,V,TRUE);
  GMATRIX_MULTIPLY_COPY_EXTENDED(MatAux2,U,FALSE,MatAux1,FALSE);
  GMATRIX_SUBSTRACT(A,MatAux2);
  printf("\nsum(sum(A-U*S*V')) = %e",GMATRIX_SUMENTRIES(A));
  Pause();


  GMATRIX_SETSIZE(A,4,4);
  GMATRIX_ORTHOGONAL(A,1);
  GMATRIX_SVD(U,S,V,A,TRUE);
  printf("\n**** GMATRIX_SVD of an orthogonal matrix 4x4");
  GMATRIX_PRINT(A);
  GMATRIX_PRINT(U);
  GMATRIX_PRINT(V);
  GMATRIX_PRINT(S);
    GMATRIX_MULTIPLY_COPY_EXTENDED(MatAux1,S,FALSE,V,TRUE);
  GMATRIX_MULTIPLY_COPY_EXTENDED(MatAux2,U,FALSE,MatAux1,FALSE);
  GMATRIX_SUBSTRACT(A,MatAux2);
  printf("\nsum(sum(A-U*S*V')) = %e",GMATRIX_SUMENTRIES(A));

  Pause();
 
  GMATRIX_SETSIZE(A,4,2);
  GMATRIX_RANDN(A);
  GMATRIX_SVD(U,S,V,A,TRUE);
  printf("\n**** GMATRIX_SVD of a random matrix 4x2");
  GMATRIX_PRINT(A);
  GMATRIX_PRINT(U);
  GMATRIX_PRINT(V);
  GMATRIX_PRINT(S);
  GMATRIX_MULTIPLY_COPY_EXTENDED(MatAux1,S,FALSE,V,TRUE);
  GMATRIX_MULTIPLY_COPY_EXTENDED(MatAux2,U,FALSE,MatAux1,FALSE);
  GMATRIX_SUBSTRACT(A,MatAux2);
  printf("\nsum(sum(A-U*S*V')) = %e",GMATRIX_SUMENTRIES(A));

  Pause();

  GMATRIX_SETSIZE(A,2,4);
  GMATRIX_RANDN(A);
  GMATRIX_SVD(U,S,V,A,TRUE);
  printf("\n**** GMATRIX_SVD of a random matrix 2x4");
  GMATRIX_PRINT(A);
  GMATRIX_PRINT(U);
  GMATRIX_PRINT(V);
  GMATRIX_PRINT(S);
  GMATRIX_MULTIPLY_COPY_EXTENDED(MatAux1,S,FALSE,V,TRUE);
  GMATRIX_MULTIPLY_COPY_EXTENDED(MatAux2,U,FALSE,MatAux1,FALSE);
  GMATRIX_SUBSTRACT(A,MatAux2);
  printf("\nsum(sum(A-U*S*V')) = %e",GMATRIX_SUMENTRIES(A));

}

void Eval_Cholesky(void)
{
  GMATRIX_DECLARE(L, 6, 6);
  GMATRIX_DECLARE(A, 6, 6);
  GMATRIX_DECLARE(MatAux1, 6, 6);
  GMATRIX_DECLARE(MatAux2, 6, 6);

  GMATRIX_SETSIZE(MatAux1,4,4);
  GMATRIX_RAND(MatAux1);
  GMATRIX_MULTIPLY_COPY_EXTENDED(A,MatAux1,TRUE,MatAux1,FALSE);
  GMATRIX_CHOLESKY(L,A);
  printf("\n**** GMATRIX_CHOLESKY of a rand matrix");
  GMATRIX_PRINT(A);
  GMATRIX_PRINT(L);
  GMATRIX_TRANSPOSE_COPY(MatAux2,L);
  GMATRIX_MULTIPLY_COPY(MatAux1,L,MatAux2);
  GMATRIX_SUBSTRACT_COPY(MatAux2,A,MatAux1);
  GMATRIX_PRINT(MatAux2);
  
}

void Eval_Submatrix(void)
{
  GMATRIX_DECLARE(Matrix1, 8, 8);
  GMATRIX_DECLARE(Matrix2, 2, 3);

  GMATRIX_RANDN(Matrix1);
  GMATRIX_SUMLINCOL(Matrix2);
  GMATRIX_PRINT(Matrix1);
  GMATRIX_PRINT(Matrix2);
  printf("\nGMATRIX_SUBMATRIX_COPY(Matrix1,2,5,Matrix2)");
  GMATRIX_SUBMATRIX_COPY(Matrix1,2,5,Matrix2);
  GMATRIX_PRINT(Matrix1);
  Pause();
  printf("\nGMATRIX_SUBMATRIX_COPY_EXTENDED(Matrix1,5,2,Matrix2,1)");
  GMATRIX_SUBMATRIX_COPY_EXTENDED(Matrix1,5,2,Matrix2,1);
  GMATRIX_PRINT(Matrix1);
  Pause();

  printf("\nGMATRIX_SUBMATRIX_FILL(Matrix1,1,6,1,6,-2.0)");
  GMATRIX_SUBMATRIX_FILL(Matrix1,1,6,1,6,-2.0);
  GMATRIX_PRINT(Matrix1);
  Pause();

  printf("\nGMATRIX_SUBMATRIX_MULTIPLY_CONST(Matrix1,1,7,1,7,-1.0)");
  GMATRIX_SUBMATRIX_MULTIPLY_CONST(Matrix1,1,7,1,7,-1.0);
  GMATRIX_PRINT(Matrix1);
  Pause();
}

void Eval_Alloc(void)
{
  GMATRIX_DECLARE(Matrix, 6, 8);
  PGMATRIX pMatrix;

  pMatrix = PGMATRIX_ALLOC(6,8);

  GMATRIX_RANDN(Matrix);
  PGMATRIX_COPY(pMatrix,&Matrix);
  GMATRIX_INFO(Matrix);
  GMATRIX_PRINT(Matrix);
  Pause();

  PGMATRIX_FREE(pMatrix);

}

void Eval_Arithmetic(void)
{
  GMATRIX_DECLARE(Matrix1, 2, 4);
  GMATRIX_DECLARE(Matrix2, 2, 4);
  GMATRIX_DECLARE(Matrix3, 4, 4);
  GMATRIX_DECLARE(Matrix4, 4, 4);
  GMATRIX_DECLARE(Result, 10, 10);
  
  GMATRIX_RANDN(Matrix1);
  GMATRIX_RANDN(Matrix2);
  GMATRIX_ORTHOGONAL(Matrix3,-2);
  GMATRIX_ORTHOGONAL(Matrix4,2);

  GMATRIX_PRINT(Matrix1);
  GMATRIX_PRINT(Matrix2);
  GMATRIX_ADD_COPY(Result,Matrix1,Matrix2);
  printf("\nResult = Matrix1 + Matrix2");
  GMATRIX_PRINT(Result);
  printf("\nResult += Result");
  GMATRIX_ADD(Result,Result);
  GMATRIX_PRINT(Result);

  Pause();

  GMATRIX_PRINT(Matrix1);
  GMATRIX_PRINT(Matrix2);
  GMATRIX_SUBSTRACT_COPY(Result,Matrix1,Matrix2);
  printf("\nResult = Matrix1 - Matrix2");
  GMATRIX_PRINT(Result);
  printf("\nResult -= Result");
  GMATRIX_SUBSTRACT(Result,Result);
  GMATRIX_PRINT(Result);

  Pause();

  GMATRIX_PRINT(Matrix1);
  GMATRIX_PRINT(Matrix3);
  GMATRIX_MULTIPLY_COPY(Result,Matrix1,Matrix3);
  printf("\nResult = Matrix1 * Matrix3");
  GMATRIX_PRINT(Result);
  printf("\nGMATRIX_ZEROES(Result)");
  GMATRIX_ZEROES(Result);
  printf("\nResult += Matrix1 * Matrix3");
  GMATRIX_MULTIPLY_ADD(Result,Matrix1,Matrix3);
  GMATRIX_PRINT(Result);
  printf("\nResult -= Matrix1 * Matrix3");
  GMATRIX_MULTIPLY_SUBSTRACT(Result,Matrix1,Matrix3);
  GMATRIX_PRINT(Result);
}

void Eval_Constant(void)
{ 
  GMATRIX_DECLARE(Matrix2, 2, 3);
  
  GMATRIX_RANDN(Matrix2);

  GMATRIX_PRINT(Matrix2);
  GMATRIX_ADD_CONST(Matrix2,1.0);
  GMATRIX_PRINT(Matrix2);
  GMATRIX_MULTIPLY_CONST(Matrix2,-1.0);
  GMATRIX_PRINT(Matrix2);
}

void Eval_Inverse(void)
{
  GMATRIX_DECLARE(Matrix1, 1, 1);
  GMATRIX_DECLARE(Matrix2, 2, 2);
  GMATRIX_DECLARE(Matrix3, 3, 3);
  GMATRIX_DECLARE(Matrix7, 7, 7);

  GMATRIX_DECLARE(MatrixInverse, 10, 10);
  
  GMATRIX_RANDN(Matrix1);
  GMATRIX_RANDN(Matrix2);
  GMATRIX_RANDN(Matrix3);
//  GMATRIX_ORTHOGONAL(Matrix7,-2);
  GMATRIX_WILKINSON(Matrix7);

  GMATRIX_PRINT(Matrix1);
  GMATRIX_INVERSE_COPY(MatrixInverse,Matrix1);
  GMATRIX_PRINT(MatrixInverse);
  
  printf("\n");
  GMATRIX_PRINT(Matrix2);
  GMATRIX_INVERSE_COPY(MatrixInverse,Matrix2);
  GMATRIX_PRINT(MatrixInverse);

  printf("\n");
  GMATRIX_PRINT(Matrix3);
  GMATRIX_INVERSE_COPY(MatrixInverse,Matrix3);
  GMATRIX_PRINT(MatrixInverse);

  printf("\n");
  GMATRIX_PRINT(Matrix7);
  GMATRIX_INVERSE_COPY(MatrixInverse,Matrix7);
  GMATRIX_PRINT(MatrixInverse);

}


void Eval_Determinant(void)
{
  GMATRIX_DECLARE(Matrix1, 1, 1);
  GMATRIX_DECLARE(Matrix2, 2, 2);
  GMATRIX_DECLARE(Matrix3, 3, 3);
  GMATRIX_DECLARE(Matrix,  5, 5);
  GMATRIX_DECLARE(DummyMatrix, 5, 5);

  GMATRIX_RANDN(Matrix1);
  GMATRIX_RANDN(Matrix2);
  GMATRIX_RANDN(Matrix3);
  
  GMATRIX_PRINT(Matrix1);
  printf("\nDeterminant = %e",GMATRIX_DETERMINANT(Matrix1,DummyMatrix));

  GMATRIX_PRINT(Matrix2);
  printf("\nDeterminant = %e",GMATRIX_DETERMINANT(Matrix2,DummyMatrix));
  printf("\nDeterminant2 = %e",GMATRIX_DETERMINANT2(Matrix2));

  GMATRIX_PRINT(Matrix3);
  printf("\nDeterminant = %e",GMATRIX_DETERMINANT(Matrix3,DummyMatrix));
  printf("\nDeterminant3 = %e",GMATRIX_DETERMINANT3(Matrix3));

  GMATRIX_IDENTITY(Matrix);
  GMATRIX_PRINT(Matrix);
  printf("\n\nDeterminant of GMATRIX_IDENTITY(Matrix) (5x5)= %f",GMATRIX_DETERMINANT(Matrix,DummyMatrix));

  GMATRIX_WILKINSON(Matrix);
  GMATRIX_PRINT(Matrix);
  printf("\nDeterminant of PGMATRIX_WILKINSON(Matrix) (5x5)= %f\n",GMATRIX_DETERMINANT(Matrix,DummyMatrix));

  GMATRIX_DATA(Matrix2,1,1) = -0.0503;
  GMATRIX_DATA(Matrix2,1,2) = 0.9248;
  GMATRIX_DATA(Matrix2,2,1) = 1.9037;
  GMATRIX_DATA(Matrix2,2,2) = 0.3334;
  GMATRIX_PRINT(Matrix2);
  printf("\nDeterminant = %e",GMATRIX_DETERMINANT(Matrix2,DummyMatrix));
  printf("\nDeterminant2 = %e",GMATRIX_DETERMINANT2(Matrix2));

}

void Eval_Trace(void)
{
  GMATRIX_DECLARE(Matrix, 6, 6);

  GMATRIX_RANDN(Matrix);

  GMATRIX_PRINT(Matrix);
  printf("\nTrace = %f",GMATRIX_TRACE(Matrix));
}

void Eval_Transpose(void)
{
  GMATRIX_DECLARE(Matrix, 4, 2);
  GMATRIX_DECLARE(MatrixTranspose, 2, 4);

  GMATRIX_RANDN(Matrix);
  GMATRIX_PRINT(Matrix);
  
  GMATRIX_TRANSPOSE_COPY(MatrixTranspose,Matrix);
  GMATRIX_PRINT(MatrixTranspose);

}

void Eval_Create(void)
{

  GMATRIX_DECLARE(LocalMatrix, 4, 2);
  GMATRIX_DECLARE(Idendity, 4, 4);
  GMATRIX_DECLARE(Zeroes, 1, 2);
  GMATRIX_DECLARE(Wilkinson, 7, 7);
  GMATRIX_DECLARE(Orthogonal, 5, 5);

  GMATRIX_RAND(LocalMatrix);

  GMATRIX_PRINT(LocalMatrix);
  GMATRIX_PRINT_EXP(LocalMatrix);
  printf("\nGMATRIX_MEAN(LocalMatrix) = %f",GMATRIX_MEAN(LocalMatrix));
  printf("\nGMATRIX_VARIANCE(LocalMatrix) = %f",GMATRIX_VARIANCE(LocalMatrix));
  
  GMATRIX_RANDN(GlobalMatrix);
  GMATRIX_PRINT(GlobalMatrix);
  GMATRIX_PRINT_EXP(GlobalMatrix);
  printf("\nGMATRIX_MEAN(GlobalMatrix) = %f",GMATRIX_MEAN(GlobalMatrix));
  printf("\nGMATRIX_VARIANCE(GlobalMatrix) = %f",GMATRIX_VARIANCE(GlobalMatrix));

  GMATRIX_IDENTITY(Idendity);
  GMATRIX_PRINT(Idendity);

  GMATRIX_ZEROES(Zeroes);
  GMATRIX_PRINT(Zeroes);

  GMATRIX_WILKINSON(Wilkinson);
  GMATRIX_PRINT(Wilkinson);

  GMATRIX_ORTHOGONAL(Orthogonal,2);
  GMATRIX_PRINT(Orthogonal);
}

int main(int argc, char* argv[])
{
	Eval_Create();
//	Eval_Transpose();
//	Eval_Trace();
//	Eval_Determinant();
//	Eval_Inverse();
//	Eval_Arithmetic();
//	Eval_Alloc();
//	Eval_Submatrix();
//	Eval_Cholesky();
//	Eval_SVD();
//	Eval_Pseudoinverse();
//	Eval_Rank();
	printf("\n");
	return 0;
}

